/*
  Underware - Pong demo
  Copyright (C) 2003 Vincent Caron <v.caron@zerodeux.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <getopt.h>

#include <SDL.h>
#include <AudioOpenAL.h>

#include "PongDrawArea.h"
#include "PongGameState.h"

int main(int argc, char** argv)
{
  int fullscreen = 0;
  int width = 640;
  int height = 480;

  int c;

  while (1) {
    int option_index = 0;
    static struct option long_options[] = {
      {"fullscreen", 1, 0, 0},
      {0, 0, 0, 0}
    };

    c = getopt_long (argc, argv, "f",
		     long_options, &option_index);
    if (c == -1)
      break;

    switch (c) {
    case 'f':
      fullscreen = 1;
      break;

    default:
      printf ("?? getopt returned character code 0%o ??\n", c);
    }
  }


  if (SDL_Init (SDL_INIT_VIDEO) != 0) {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError ());
    return (1);
  }
  atexit(SDL_Quit);

  SDL_WM_SetCaption("underpong", NULL);
  SDL_EnableUNICODE(1);

  /* Information about the current video settings. */
  const SDL_VideoInfo* info = NULL;
  /* Color depth in bits of our window. */
  int bpp = 0;
  /* Flags we will pass into SDL_SetVideoMode. */
  int flags = 0;
  
  /* Let's get some video information. */
  info = SDL_GetVideoInfo( );

  if( !info ) {
    /* This should probably never happen. */
    fprintf(stderr, "Video query failed: %s\n", SDL_GetError( ) );
    return 1;
  }

  /*
   * Set our width/height to 640/480 (you would
   * of course let the user decide this in a normal
   * app). We get the bpp we will request from
   * the display. On X11, VidMode can't change
   * resolution, so this is probably being overly
   * safe. Under Win32, ChangeDisplaySettings
   * can change the bpp.
   */
  bpp = info->vfmt->BitsPerPixel;

  /*
   * Now, we want to setup our requested
   * window attributes for our OpenGL window.
   * We want *at least* 5 bits of red, green
   * and blue. We also want at least a 16-bit
   * depth buffer.
   *
   * The last thing we do is request a double
   * buffered window. '1' turns on double
   * buffering, '0' turns it off.
   *
   * Note that we do not use SDL_DOUBLEBUF in
   * the flags to SDL_SetVideoMode. That does
   * not affect the GL attribute state, only
   * the standard 2D blitting setup.
   */
  SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

  flags = SDL_OPENGL;
  if(fullscreen)
    flags |= SDL_FULLSCREEN;

  SDL_Surface* screen;

  /*
   * Set the video mode
   */
  if( (screen = SDL_SetVideoMode( width, height, bpp, flags )) == 0 ) {
    /* 
     * This could happen for a variety of reasons,
     * including DISPLAY not being set, the specified
     * resolution not being available, etc.
     */
    fprintf(stderr, "Video mode set failed: %s\n", SDL_GetError( ) );
    return 1;
  }

  SDL_ShowCursor (SDL_DISABLE);

  AudioOpenAL* audio = AudioOpenAL::instance();
  audio->playGameMusic();

  int done = 0;
  Uint8 *keys;
  SDL_Event event;
  Uint32 lastUpdate = 0;

  PongDrawArea* scene = new PongDrawArea();
  PongGameState* state = new PongGameState();
  scene->LinkGameState(state);

  while (!done) {
    Uint32 now = SDL_GetTicks();
    Uint32 delta = now - lastUpdate;
    int expose = 0;

    if (SDL_PollEvent (&event) != 0) {
      switch(event.type) {
      case SDL_KEYUP:
	switch(event.key.keysym.sym) {
	case SDLK_ESCAPE:
	  done = 1;
	  break;
	default:
	  break;
	}
	break;
      case SDL_VIDEOEXPOSE:
	expose = 1;
	break;
      default:
	break;
      }
    }

    /* 100 fps is enough. */
    if(expose || delta > 10) {
      /* keyboard handling */
      keys = SDL_GetKeyState (NULL);
      if(keys[SDLK_DOWN]) {
	state->PaddleDown(PongGameState::cPlayerWithKeyboard, delta);
      }
      if(keys[SDLK_UP]) {
	state->PaddleUp(PongGameState::cPlayerWithKeyboard, delta);
      }

      {
	int x, y;
	SDL_GetMouseState(&x, &y);
	state->PaddlePos(PongGameState::cPlayerWithMouse, ((float)x) / width, ((float)y) / height);
      }

      state->UpdateBall(delta);

      scene->DrawGame();

      lastUpdate = now;
    } else {
      SDL_Delay(delta);
    }
  }

  delete audio;
}
