/*
  Underware - Pong demo
  Copyright (C) 2003 Vincent Caron <v.caron@zerodeux.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <math.h>

#include "AudioOpenAL.h"
#include "PongGameState.h"

/* Returns a random float in [-1,1]
 */
float randf()
{
  return (float)rand() * 2.f / RAND_MAX - 1.f;
}


const float PongGameState::cPaddleHWidth     = .025f;
const float PongGameState::cPaddleHLength    = .2;
const float PongGameState::cPaddleDeflection = .5f;
const float PongGameState::cBallHSize        = .025f;
const float PongGameState::cBallAcceleration = .001f;
const int PongGameState::cPlayerWithKeyboard = 0;
const int PongGameState::cPlayerWithMouse    = 1;


PongGameState::PongGameState() :
  mPlayerYPos(2)
{
  Init();
};

void PongGameState::Init()
{
  mState = eStopped;
  mCredits = 3;
  mPings = 0;
  mPlayerYPos[0] = 0.f;
  mPlayerYPos[1] = 0.f;
  mBallXPos = 0.f;
  mBallYPos = 0.f;
  mBallXSpeed = cBallAcceleration;
  mBallYSpeed = cBallAcceleration;
}


const float cKeyboardSpeed    = 0.005; // number of pixels per centiseconds

void PongGameState::PaddleUp(int player, Uint32 delta)
{
  /* Keyboard control for the paddle
   */
  float& y = mPlayerYPos[player];
  float dy = cKeyboardSpeed * delta;
  
  y += dy;

  if (y > 1.f) y = 1.f;
}

void PongGameState::PaddleDown(int player, Uint32 delta)
{
  /* Keyboard control for the paddle
   */
  float& y = mPlayerYPos[player];
  float dy = cKeyboardSpeed * delta;
  
  y -= dy;

  if (y < -1.f) y = -1.f;
}

void PongGameState::PaddlePos(int player, float x, float y)
{
  /* Normalize these coordinates (from pixels to [-1, 1])
   */
  x = x * 2.f - 1.f;
  y = 1.f - y * 2.f;

  /* Make sure it gets in the [-1, 1] range
   */
  if (y >= -1.f && y <= 1.f)
    mPlayerYPos[cPlayerWithMouse] = y;
}

void PongGameState::UpdateBall (Uint32 delta)
{
  /* Let's move the ball
   */
  mBallXPos += mBallXSpeed * delta;
  mBallYPos += mBallYSpeed * delta;

  ALfloat pos[] = { mBallXPos, mBallYPos, 0.0 };
  AudioOpenAL* audio = AudioOpenAL::instance();

  /* Check collisions with top and bottom screen edges
   */
  if (mBallYPos >= 1.f)
  {
    mBallYSpeed = -mBallYSpeed;
    mBallYPos -= mBallYPos - 1.f;
    audio->playSound(Bounce, pos);
  }
  if (mBallYPos <= -1.f)
  {
    mBallYSpeed = -mBallYSpeed;
    mBallYPos -= mBallYPos + 1.f;
    audio->playSound(Bounce, pos);
  }

  /* Check collisions with paddles
   */
  int paddle = -1;
  if (mBallXPos < -1.f) paddle = 0; else
  if (mBallXPos > 1.f)  paddle = 1;

  if (paddle >= 0)
  {
    /* Compute paddle and ball position in the drawing coordinate system
     * (same job as in PongDrawArea)
     */
    float paddle_y = mPlayerYPos[paddle] * (1.f - cPaddleHLength);
    float ball_y   = mBallYPos * (1.f - cBallHSize);
    float coverage = (ball_y - paddle_y) / (cPaddleHLength + cBallHSize);

    /* Check for game end
     */
    if (coverage < -1.f || coverage > 1.f)
    {
      Init();
      return;
    }
    mPings++;

    /* Compute reflection
     */
    float sspeed = mBallXSpeed * mBallXSpeed + mBallYSpeed * mBallYSpeed;
    sspeed *= (1.f + cBallAcceleration);

//    NGL_OUT("sspeed: %f (%f, %f) -> ", sspeed, mBallXSpeed, mBallYSpeed);
    mBallYSpeed = mBallYSpeed * (.9f + fabs(coverage) * .6f /** cPaddleDeflection*/);
    mBallXSpeed = (mBallXSpeed > 0.f ? -1.f : 1.f) * sqrt(sspeed - mBallYSpeed * mBallYSpeed);
//    NGL_OUT("(%f, %f)\n", mBallXSpeed, mBallYSpeed);
    mBallXPos = (mBallXPos > 0.f ? 2.f : -2.f) - mBallXPos;

    audio->playSound(Bounce, pos);
  }
}
